package jp.co.isid.elportal.fileconvert.service.convert;

import java.io.File;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

import jp.co.isid.elportal.base.cifs.CifsAccessor;
import jp.co.isid.elportal.base.cifs.CifsAccessorFactory;
import jp.co.isid.elportal.base.cifs.exception.CifsApplicationException;
import jp.co.isid.elportal.base.log.Logger;
import jp.co.isid.elportal.fileconvert.command.CommandException;
import jp.co.isid.elportal.fileconvert.command.JpgPngToPdfCommand;
import jp.co.isid.elportal.fileconvert.command.TiffToJpegCommand;
import jp.co.isid.elportal.fileconvert.command.TiffToPdfCommand;
import jp.co.isid.elportal.fileconvert.consts.CommonConstants.CifsType;
import jp.co.isid.elportal.fileconvert.util.StringUtil;
import jp.co.isid.soa.framework.service.exception.ExceptionHandler;
import jp.co.isid.soa.framework.service.exception.ServiceException;

/**
 *
 * バッチプログラムから呼び出される、TIFFファイル変換サービスです。
 *
 *
 */
@WebService(
        serviceName = "convertServiceForBatch",
        targetNamespace = "http://convert.model.file_convert.elportal.isid.co.jp/"
        )
public class ConvertServiceForBatch {

    /** 変換後のファイル区切り文字 **/
    private static final String RETURN_FILE_SEPARATOR = "/";
    /** 変換前のファイル区切り文字 **/
    private static final String REPLACE_FILE_SEPARATOR = "\\\\";
    /** 画像種別：JPEG **/
    private static final String IMG_TYPE_J = "5";
    /** 画像種別：PNG **/
    private static final String IMG_TYPE_P = "6";


    /**
     * TIFFファイルをPDFファイルへ変換するサービスです。
     *
     * @param tiffFile TIFFファイルへの相対パス
     * @return PDFファイルへの相対パス
     * @throws ServiceException 変換に失敗した場合。
     */
    @WebMethod
    @WebResult(name = "pdfFile")
    public String convertToPdf(
            @WebParam(name = "tiffFile")String tiffFile)
                    throws ServiceException {
        try {
            String pdfPath = "";
            File file = new File(editPath(tiffFile));

            String fileType = file.getName().substring(13, 14);
            if (IMG_TYPE_J.equals(fileType) || IMG_TYPE_P.equals(fileType)) {
                // ファイル区分がJPEGまたはPNGの場合
                JpgPngToPdfCommand command = new JpgPngToPdfCommand();
                pdfPath = command.execute(file);
            } else {
                // それ以外の場合はtiffの場合
                TiffToPdfCommand command = new TiffToPdfCommand();
                pdfPath = command.execute(file);
            }

            return this.makeRelativePath(pdfPath);
        } catch (CommandException e) {
            ExceptionHandler.getInstance().handleException(e);
            throw new AssertionError();
        }
    }

    /**
     * TIFFファイルをJPEG（サムネイル用）に変換するファイルです。
     * 内部で別プロセスを立ち上げます。（TiffToJpegCommand参照）
     *
     * @param tiffFile TIFFファイルへの相対パス
     * @return PDFファイルへの相対パス
     * @throws ServiceException 変換に失敗した場合。
     */
    @WebMethod
    @WebResult(name = "jpegFile")
    public String convertToJpeg(@WebParam(name = "tiffFile")String tiffFile)
            throws ServiceException {
        try {
            TiffToJpegCommand tiffCommand = new TiffToJpegCommand();
            String jpgPath = tiffCommand.execute(new File(editPath(tiffFile)));
            return this.makeRelativePath(jpgPath);
        } catch (CommandException e) {
            ExceptionHandler.getInstance().handleException(e);
            throw new AssertionError();
        }
    }

    /**
     * パスを編集する。<br>
     *
     * パスのプレフィックスが指定されていれば、<br>
     * パスにプレフィックスを追加し、UNC形式で返却する。
     *
     * @param path パス
     * @return String 編集されたパス
     * @throws CommandException ファイルがなかった場合
     */
    private String editPath(String path) throws CommandException {
        String fullPath = null;
        try {
            CifsAccessor cifs = CifsAccessorFactory.getInstance().getAccessor(CifsType.TIFF.getName());
            fullPath = cifs.getPath(path);
        } catch (CifsApplicationException e) {
            Logger.error("ファイルパスの取得に失敗しました：" + path, e);
            throw new CommandException("ファイルパスの取得に失敗しました：" + path, e);
        }

        return fullPath;
    }

    /**
     * 相対パスの編集をする。<br>
     *
     * 第一引数pathから第二引数saveFolderを取り除く。
     *
     * @param path パス
     * @return String 編集されたパス
     */
    private String makeRelativePath(String path) {
        if (StringUtil.isBlank(path)) {
            return path;
        }

        // File.separator ⇒ "/"に変換
        return path.replaceAll(REPLACE_FILE_SEPARATOR, RETURN_FILE_SEPARATOR);
    }
}
