/*
 * Copyright(c) 2010 Information Services International-Dentsu, Ltd.
 */
package jp.co.isid.elportal.fileconvert.util;

import java.io.IOException;
import java.util.List;

import jp.co.isid.elportal.base.config.ParameterCatalog;
import jp.co.isid.elportal.base.exception.SystemException;
import jp.co.isid.elportal.base.log.Logger;
import jp.co.isid.elportal.fileconvert.service.convert.ArticleDataPrintVO;
import jp.co.isid.elportal.fileconvert.service.convert.ArticleDetailVO;
import jp.co.isid.elportal.fileconvert.service.convert.ArticleVO;

import com.itextpdf.text.Chunk;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.draw.DottedLineSeparator;
import com.itextpdf.text.pdf.draw.LineSeparator;

/**
 * <P>
 * 記事データ印刷Utilityクラス
 * </P>
 *
 * @version $Revision: $ $Date: $
 */

public abstract class ArtclHdlnListPrntUtility {

    /** BASE_FONT_NAME */
    private static final String BASE_FONT_NAME = ParameterCatalog.instance().getParameter("FileConvertParam", "fontNameGothic");
    /** BASE_FONT_TYPE */
    private static final String BASE_FONT_TYPE = BaseFont.IDENTITY_H;

    /** 記事データヘッダメッセージ */
    private static final String HEADER_MSG = "　◎以下の記事データリストはＥＬＮＥＴから提供しています。\r\n　記事データ　";

    /** 記事データヘッダメッセージ（件数） */
    private static final String HEADER_MSG_COUNT = "件　　";

    /** 原文提供不可マーク */
    private static final String OLGNO_MAR = "◇";
    /** 原文提供不可メッセージ */
    private static final String OLGNO_MSG = "【原文提供不可】";

    /** 改行コード */
    private static final String BR = "\r\n";

    /**
     * コンストラクタ
     */
    private ArtclHdlnListPrntUtility() {
    }

    /**
     * ボディヘッダー取得
     *
     * @param printvo 印刷情報VO
     * @return ボディヘッダー情報
     */
    public static Paragraph getBodyHeader(ArticleDataPrintVO printvo) {

        try {

            // ボディ部フォント
            BaseFont bf = BaseFont.createFont(BASE_FONT_NAME, BASE_FONT_TYPE, false);
            Font font8 = new Font(bf, 8);
            Paragraph pdataHead = new Paragraph(HEADER_MSG + printvo.getAcDataCount() + HEADER_MSG_COUNT + toBlank(printvo.getPrintDateJp()), font8);
            pdataHead.setSpacingAfter(10.0f);

            return pdataHead;

        } catch (IOException ioe) {
            Logger.error(ioe.getMessage(), ioe);
            throw new SystemException(ioe);
        } catch (DocumentException de) {
            Logger.error(de.getMessage(), de);
            throw new SystemException(de);
        }
    }

    /**
     * ボディ部を取得
     *
     * @param acvo ArticleDetailVO
     * @return ボディ部
     */
    public static Paragraph getBodyRow(ArticleDetailVO acvo) {

        try {

            // ボディ部フォント
            BaseFont bf = BaseFont.createFont(BASE_FONT_NAME, BASE_FONT_TYPE, false);
            Font font8 = new Font(bf, 8);
            Font font9 = new Font(bf, 9);
            Font font10 = new Font(bf, 10);
            Font font10Underline = new Font(bf, 10, Font.UNDERLINE);
            Font font10Red = new Font(bf, 10, Font.NORMAL, new BaseColor(0xE1, 0x6D, 0x5D));

            // ボディ用テーブル
            PdfPTable  table = new PdfPTable (2);
            table.setWidths(new int[]{60, 800});
            table.setWidthPercentage(100);

            // 番号
            PdfPCell  cell;
            cell = new PdfPCell (new Paragraph(acvo.getListNo() + "　", font9));
            cell.setBorder(Rectangle.NO_BORDER);
            cell.setLeading(1.0f, 1.7f);
            cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
            table.addCell(cell);

            // 記事見出し
            Paragraph ptitle = new Paragraph();
            // 原文提供不可の場合
            if (acvo.isOrgNoFlg()) {
                ptitle.add(new Chunk(OLGNO_MAR, font10));
                ptitle.add(new Chunk(acvo.getActitle(), font10Underline));
                ptitle.add(new Chunk(OLGNO_MSG, font10Red));
            } else {
                ptitle.add(new Chunk(acvo.getActitle(), font10Underline));
            }
            // 行間の調整
            ptitle.setLeading(20.5f);

            // 空白部分
            cell = new PdfPCell (ptitle);
            cell.setBorder(Rectangle.NO_BORDER);
            cell.setLeading(1.0f, 1.5f);
            table.addCell(cell);
            cell = new PdfPCell (new Paragraph("", font9));
            cell.setBorder(Rectangle.NO_BORDER);
            table.addCell(cell);

            // 記事属性＋キーワード部分
            Paragraph pacdata = new Paragraph();
            // 記事属性
            for (String acdata : acvo.getAcdata()) {
                pacdata.add(new Chunk(BR, font8));
                pacdata.add(new Chunk(acdata, font8));
            }
            // キーワード
            if (acvo.isKeywordDisp()) {
                for (String keyword : acvo.getKeyword()) {
                    pacdata.add(new Chunk(BR, font8));
                    pacdata.add(new Chunk(keyword, font8));
                }
            }
            cell = new PdfPCell (pacdata);
            cell.setBorder(Rectangle.NO_BORDER);
            table.addCell(cell);

            // ボディ部（１行）の作成
            Paragraph pdataRow = new Paragraph();
            pdataRow.add(table);

            return pdataRow;

        } catch (IOException ioe) {
            Logger.error(ioe.getMessage(), ioe);
        } catch (DocumentException de) {
            Logger.error(de.getMessage(), de);
        }
        return null;

    }


    /**
     * 区切り線の取得
     *
     * @return 区切り線Element
     */
    public static Element getLine() {
        LineSeparator line = new LineSeparator();
        line.setLineWidth(2.0f);
        line.setLineColor(new BaseColor(0xCC, 0xCC, 0xCC));
        return line;
    }

    /**
     * 区切り線（破線）の取得
     *
     * @return 区切り線Element
     */
    public static Element getDottedLine() {
        // 区切り線（破線）
        DottedLineSeparator line2 = new DottedLineSeparator();
        line2.setLineColor(new BaseColor(0x77, 0x77, 0x77));
        line2.setOffset(-2.0f);
        return line2;
    }

    /**
     * 見出し一覧表示件数を返却します
     *
     * @param orgFlg 原文出力フラグ
     * @param acVoArray 原文記事VOの配列
     * @param detailVoList 印刷記事VOのリスト
     * @param acDataCount 見出し一覧表示件数（全部）
     * @return int 見出し一覧表示件数
     */
    public static int getAcDataCount(boolean orgFlg, ArticleVO[] acVoArray, List<ArticleDetailVO> detailVoList, int acDataCount) {
        int count = 0;

        // 見出しリストなし
        if (detailVoList == null || detailVoList.isEmpty()) {
            return count;
        }

        // 原文印刷しないならそのままの件数
        if (!orgFlg) {
            return acDataCount;
        }

        // 原文記事なしの場合はそのままの件数
        if (acVoArray == null || acVoArray.length == 0) {
            return acDataCount;
        }

        // 見出しリストから、原文なしの記事は除く
        for (ArticleDetailVO vo : detailVoList) {
            if (isOutputAcno(orgFlg, acVoArray, vo)) {
                count++;
            }
        }

        return count;
    }

    /**
     * 見出し出力するかどうか
     *
     * @param orgFlg 原文出力フラグ
     * @param acVoArray 原文VOの配列
     * @param detailVo 印刷VO
     * @return boolean true - する、false - しない
     */
    public static boolean isOutputAcno(boolean orgFlg, ArticleVO[] acVoArray, ArticleDetailVO detailVo) {

        // 原文印刷しないなら見出し出力OK
        if (!orgFlg) {
            return true;
        }

        // 原文記事が1件もなかったら見出し出力OK
        if (acVoArray == null || acVoArray.length == 0) {
            return true;
        }

        // AC番号が指定されてなかったらNG
        String acno = toBlank(detailVo.getAcno());
        if ("".equals(acno)) {
            return false;
        }

        // 原文イメージのないAC番号かどうかチェック
        boolean output = false;
        boolean contain = false;
        for (ArticleVO acVo : acVoArray) {
            if (acno.equals(acVo.getAcNo())) {
                contain = true;
                output = acVo.isOutputAcno();

                break;
            }
        }

        boolean result = false;
        if (!contain) {
            // 原文記事リストになかったら（原文提供不可なので）見出しOK
            result = true;
        } else {
            // 原文記事リストにあったら見出し出力するかどうかを見る
            result = output;
        }

        return result;
    }

    /**
     * NULLをブランクに変換します
     *
     * @param value 変換する文字列
     * @return String 変換した文字列
     */
    private static String toBlank(String value) {
        if (value == null) {
            return "";
        }

        return value;
    }
}